package main

import (
	"flag"
	"log"
	"log/syslog"
	"os"
	"strings"

	"github.com/pyke369/golang-support/uconfig"
)

func main() {
	var err error
	var config *uconfig.UConfig
	// default configuration file is ./openvpn-dm-mgt-server.conf
	configFile := flag.String("config", "openvpn-dm-mgt-server.conf", "configuration file")
	logToSyslog := flag.Bool("syslog", false, "Log to syslog")
	flag.Parse()

	// parseconfig
	if config, err = uconfig.New(*configFile); err != nil {
		log.Println(err)
		os.Exit(1)
	}

	server := NewVPNServer(config.GetString("config.openvpnPort", "127.0.0.01:5000"))
	server.vpnlogUrl = config.GetString("config.vpnLogUrl", "")
	server.mailRelay = config.GetString("config.mailRelay", "")
	server.MailFrom = config.GetString("config.mailFrom", "")
	server.CcPwnPassword = config.GetString("config.ccPwnPassword", "")
	server.pwnTemplate = config.GetString("config.pwnTemplate", "")
	server.newAsTemplate = config.GetString("config.newAsTemplate", "")
	server.cacheDir = config.GetString("config.cacheDir", "")
	server.authCa = config.GetString("config.authCa", "")
	server.otpMasterSecrets = parseConfigArray(config, "config.masterSecrets")
	if len(server.otpMasterSecrets) == 0 {
		server.otpMasterSecrets = append(server.otpMasterSecrets, "*******************")
	}

	server.syslog = false
	if *logToSyslog {
		log.SetFlags(0)
		server.syslog = true
		logWriter, e := syslog.New(syslog.LOG_NOTICE, "")
		if e == nil {
			log.SetOutput(logWriter)
			defer logWriter.Close()
		}
	}

	for _, profile := range config.GetPaths("config.profiles") {
		profileName := strings.Split(profile, ".")[2]
		ldapConf := ldapConfig{
			servers:            parseConfigArray(config, profile+".servers"),
			baseDN:             config.GetString(profile+".baseDN", ""),
			bindCn:             config.GetString(profile+".bindCn", ""),
			bindPw:             config.GetString(profile+".bindPw", ""),
			searchFilter:       config.GetString(profile+".searchFilter", ""),
			primaryAttribute:   config.GetString(profile+".primaryAttribute", ""),
			secondaryAttribute: config.GetString(profile+".secondaryAttribute", ""),
			validGroups:        parseConfigArray(config, profile+".validGroups"),
			routes:             parseConfigArray(config, profile+".routes"),
			mfaType:            config.GetString(profile+".mfa", ""),
			certAuth:           config.GetString(profile+".cert", "optionnal"),
			upgradeFrom:        config.GetString(profile+".upgradeFrom", ""),
		}
		ldapConf.addIPRange(config.GetString(profile+".IPRange", ""))

		server.ldap[profileName] = ldapConf
	}

	// time to start the listeners
	go server.Run()
	NewHTTPServer(
		config.GetString("config.httpPort", "127.0.0.01:8080"),
		config.GetString("config.httpKey", ""),
		config.GetString("config.httpCert", ""),
		config.GetString("config.httpCa", ""),
		server)
}
